/// <reference types="node" />
import { AccountInfo, PublicKey } from '@solana/web3.js';
import BN from 'bn.js';
import { AnyPublicKey, StringPublicKey } from "../../../types";
import { Borsh } from "../../../utils";
import { Account } from '../../../Account';
import { NFTPacksAccountType } from '../NFTPacksProgram';
import { Buffer } from 'buffer';
export declare enum DistributionType {
    FixedNumber = 0,
    ProbabilityBased = 1
}
declare type DistributionArgs = {
    type: DistributionType;
    value: BN;
};
export declare class Distribution extends Borsh.Data<DistributionArgs> {
    static readonly SCHEMA: any;
    type: DistributionType;
    value: BN;
}
declare type Args = {
    packSet: StringPublicKey;
    master: StringPublicKey;
    metadata: StringPublicKey;
    tokenAccount: StringPublicKey;
    maxSupply?: number;
    distribution: Distribution;
    currentSupply: number;
};
export declare class PackCardData extends Borsh.Data<Args> {
    static readonly SCHEMA: any;
    accountType: NFTPacksAccountType;
    packSet: StringPublicKey;
    master: StringPublicKey;
    metadata: StringPublicKey;
    tokenAccount: StringPublicKey;
    maxSupply?: number;
    distribution: Distribution;
    currentSupply: number;
    constructor(args: Args);
}
export declare class PackCard extends Account<PackCardData> {
    static readonly PREFIX = "card";
    constructor(pubkey: AnyPublicKey, info: AccountInfo<Buffer>);
    static isCompatible(data: Buffer): boolean;
    static getPDA(packSet: AnyPublicKey, index: number): Promise<PublicKey>;
}
export {};
